@@ -1,37 +1,31 @@
-/*
- * main.c -- main loop, and interface with user
- *
- * Copyright (c) 1988-1993 Miguel Santana
- * Copyright (c) 1995-2000 Akim Demaille, Miguel Santana
- * Copyright (c) 2007 Akim Demaille, Miguel Santana and Masayuki Hatta
- */
+/* main.c - main loop, and interface with user
+   Copyright 1988-2017 Free Software Foundation, Inc.
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3, or (at your option)
+   any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
+   02110-1301, USA.  */
 
-/*
- * This file is part of a2ps.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; see the file COPYING.  If not, write to
- * the Free Software Foundation, 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- */
 
-
 /************************************************************************/
 /*                                                                      */
 /*                      I n c l u d e   f i l e s                       */
 /*                                                                      */
 /************************************************************************/
-#include <assert.h>
+#include <config.h>
+
+#include <signal.h>
+#include <locale.h>
 
 #include "a2ps.h"
 #include "argmatch.h"
@@ -52,16 +46,15 @@
 #include "title.h"
 #include "useropt.h"
 #include "main.h"
+#include "routines.h"
 #include "lexps.h"
-#include <signal.h>
-#include "signame.h"
+#include "sig2str.h"
 #include "long-options.h"
 #include "version-etc.h"
-#include <locale.h>
 
 /* From basename.c */
-char *base_name PARAMS ((const char *path));
-
+char *base_name (const char *path);
+
 /************************************************************************/
 /*                                                                      */
 /*                 G l o b a l   d e f i n i t i o n s                  */
@@ -104,7 +97,6 @@ enum behavior behavior = b_ps;
    defined twice, see lib/confg.gperf, handling of `Options:'. */
 
 char *program_name;
-const char *program_invocation_name;
 
 /* Stores the data of liba2ps.  */
 
@@ -157,7 +149,7 @@ bool delegate_p = true;
 /*
  * --toc[=format], generate a table of content
  */
-uchar *toc = NULL;
+char *toc = NULL;
 
 /*
  * -E: style sheet to use. NULL => automated
@@ -179,7 +171,7 @@ int strip_level = 0;
  * interpret as end of line
  */
 enum eol_e end_of_line = eol_auto;
-
+
 /************************************************************************/
 /*                             Service routines                         */
 /************************************************************************/
@@ -195,12 +187,14 @@ exit_handler (void)
     unlink (sample_tmpname);
 }
 
-static RETSIGTYPE
+static void
 signal_handler (int signum)
 {
   /* Error calls exit which calls atexit which removes the files. */
+  char strsignal[SIG2STR_MAX];
+  sig2str (signum, strsignal);
   error (EXIT_FAILURE, 0,
-	 _("received signal %d: %s"), signum, strsignal (signum));
+	 _("received SIG%s"), strsignal);
 }
 
 /************************************************************************
@@ -227,8 +221,8 @@ static const int highlight_level_types[] =
 static int
 get_highlight_level (const char *option, const char *arg)
 {
-  ARGMATCH_ASSERT (highlight_level_args, highlight_level_types);
-  return XARGCASEMATCH (option, arg,
+  //ARGMATCH_VERIFY (highlight_level_args, highlight_level_types);
+  return XARGMATCH (option, arg,
 			highlight_level_args, highlight_level_types);
 }
 
@@ -305,7 +299,7 @@ static const enum behavior behavior_types[] =
   b_glob,
   b_ps
 };
-
+
 /************************************************************************/
 /*                              Interface                               */
 /************************************************************************/
@@ -326,26 +320,23 @@ list_options (struct a2ps_job *a_job, FILE *stream)
 #endif
 
 #define bool_to_string(bool) ((bool) ? _("yes") : _("no"))
-  uchar buf[256], buf2[256];
-  const char *cp = NULL;
-  uchar *ucp = NULL;
+  char buf[256], buf2[256];
+  const char *cp = NULL, *cp2 = NULL;
 
   /* Title of --list-options (%s%s is `a2ps' `version' */
   title (stream, '=', true,
 	 _("Configuration status of %s %s\n"),
-	 program_invocation_name, VERSION);
+	 program_name, VERSION);
   putc ('\n', stream);
 
   title (stream, '-', false, _("Sheets:\n"));
   fprintf (stream, _("\
-  medium          = %s%s, %s\n\
-  page layout     = %d x %d, %s\n\
+  medium          = %s, %s\n\
+  page layout     = %zu x %zu, %s\n\
   borders         = %s\n\
   file alignment  = %s\n\
-  interior margin = %d\n"),
+  interior margin = %u\n"),
 	   a_job->medium->name,
-	   (a2ps_medium_libpaper_p (job, job->medium_request)
-	    ? " (libpaper)" : ""),
 	   (a_job->orientation == portrait) ? _("portrait") : _("landscape"),
 	   a_job->columns, a_job->rows,
 	   madir_to_string (a_job->madir),
@@ -355,33 +346,33 @@ list_options (struct a2ps_job *a_job, FILE *stream)
   putc ('\n', stream);
 
   if (a_job->columns_requested > 0)
-    sprintf ((char *) buf, _("%d characters per line"),
+    sprintf ((char *) buf, _("%u characters per line"),
 	     a_job->columns_requested);
   else if (a_job->lines_requested > 0)
-    sprintf ((char *) buf, _("%d lines per page"),
+    sprintf ((char *) buf, _("%u lines per page"),
 	     a_job->lines_requested);
   else
-    sprintf ((char *) buf, _("font size is %gpt"), a_job->fontsize);
+    sprintf ((char *) buf, _("font size is %gpt"), (double) a_job->fontsize);
 
   switch (a_job->numbering)
     {
     case 0:
-      ustrcpy (buf2, _("no"));
+      strcpy (buf2, _("no"));
       break;
     case 1:
       /* number line: each line */
-      ustrcpy (buf2, _("each line"));
+      strcpy (buf2, _("each line"));
       break;
     default:
       /* number line: each %d line */
-      sprintf ((char *) buf2, _("each %d lines"), a_job->numbering);
+      sprintf ((char *) buf2, _("each %u lines"), a_job->numbering);
     }
 
   title (stream, '-', false, _("Virtual pages:\n"));
   fprintf (stream, _("\
   number lines         = %s\n\
   format               = %s\n\
-  tabulation size      = %d\n\
+  tabulation size      = %u\n\
   non printable format = %s\n"),
 	   buf2,
 	   buf,
@@ -435,9 +426,9 @@ list_options (struct a2ps_job *a_job, FILE *stream)
   if (IS_EMPTY (style_request))
     /* TRANS: a2ps -E --list=options.  Warning, this answer is also
        used for the PPD file.  Make it compatible with both.  */
-    ustrcpy (buf, _("selected automatically"));
+    strcpy (buf, _("selected automatically"));
   else
-    ustrcpy (buf, style_request);
+    strcpy (buf, style_request);
   title (stream, '-', false, _("Pretty-printing:\n"));
   fprintf (stream, _("\
   style sheet     = %s\n\
@@ -453,26 +444,26 @@ list_options (struct a2ps_job *a_job, FILE *stream)
    */
 
   /* Make a nice message to tell where the output is sent */
-  ucp = a2ps_flag_destination_to_string (a_job);
+  cp2 = a2ps_flag_destination_to_string (a_job);
 
   /* Make a nice message to tell what version control is used */
   switch (a_job->backup_type)
     {
-    case none:
+    case no_backups:
       cp = _("never make backups");
       break;
 
-    case simple:
+    case simple_backups:
       cp = _("simple backups of every file");
       break;
 
-    case numbered_existing:
+    case numbered_existing_backups:
       /* appears in a2ps --version-=existing --list=defaults */
       cp = _("numbered backups of files already numbered,\n\
                             and simple of others");
       break;
 
-    case numbered:
+    case numbered_backups:
       cp = _("numbered backups of every file");
       break;
     }
@@ -482,9 +473,8 @@ list_options (struct a2ps_job *a_job, FILE *stream)
   destination     = %s\n\
   version control = %s\n\
   backup suffix   = %s\n"),
-	   ucp, cp, simple_backup_suffix);
+	   cp2, cp, simple_backup_suffix);
   putc ('\n', stream);
-  free (ucp);
 
   /*
    * PostScript report.
@@ -497,7 +487,7 @@ list_options (struct a2ps_job *a_job, FILE *stream)
   Printer Description (PPD) = %s\n\
   default PPD               = %s\n\
   page label format         = %s\n\
-  number of copies          = %d\n\
+  number of copies          = %u\n\
   sides per sheet           = %s\n\
   page device definitions   = "),
 	   a_job->status->magic_number,
@@ -526,7 +516,7 @@ list_options (struct a2ps_job *a_job, FILE *stream)
    */
   title (stream, '-', false, _("Internals:\n"));
   fprintf (stream, _("\
-  verbosity level     = %d\n\
+  verbosity level     = %u\n\
   file command        = %s\n\
   library path        = \n"),
 	   msg_verbosity,
@@ -565,8 +555,8 @@ spy_user (struct a2ps_job *a_job, FILE * stream)
      of my time.  */
   fputs ("SPY-BEGIN\n", spy);
   fputs ((char *) expand_user_string (job, CURRENT_FILE (job),
-				      (const uchar *) "Debugging info",
-				      (const uchar *) "%V was called with #!$|| |\n\n"),
+				      "Debugging info",
+                                      "%V was called with #!$|| |\n\n"),
 	 spy);
 
   list_options (a_job, spy);
@@ -577,7 +567,7 @@ spy_user (struct a2ps_job *a_job, FILE * stream)
   /* Yes, I know, there are certainly better means.  Just teach them
      to me...  */
   fclose (spy);
-  fopen (spyname, "r");
+  spy = fopen (spyname, "r");
   if (!spy)
     error (1, errno, _("cannot open file `%s'"), quotearg (spyname));
 
@@ -659,7 +649,7 @@ is sent to the default printer.  An output file may be specified with -o.\n\
 Mandatory arguments to long options are mandatory for short options too.\n\
 Long options marked with * require a yes/no argument, corresponding\n\
 short options stand for `yes'.\n"),
-	   program_invocation_name);
+	   program_name);
 
   /*
    * Does not print, and exits with success
@@ -825,7 +815,7 @@ News, updates and documentation: visit http://www.gnu.org/software/a2ps/.\n"));
 
   exit (status);
 }
-
+
 
 /*----------------------------------------------------------------.
 | Handle the options that a2ps understands (not liba2ps) Return 1 |
@@ -884,8 +874,8 @@ handle_a2ps_option (int option, char *optional_arg)
       break;
 
     case 145:
-      ARGMATCH_ASSERT (behavior_args, behavior_types);
-      behavior = XARGCASEMATCH ("--list", optional_arg,
+      //ARGMATCH_VERIFY (behavior_args, behavior_types);
+      behavior = XARGMATCH ("--list", optional_arg,
 				behavior_args, behavior_types);
       break;
 
@@ -909,7 +899,7 @@ handle_a2ps_option (int option, char *optional_arg)
 
     case 167:			/* --toc[=toc format]           */
       /* If no argument is given, use #{toc}. */
-      xustrcpy (toc, optional_arg ? optional_arg : "#{toc}");
+      xstrcpy (toc, optional_arg ? optional_arg : "#{toc}");
       break;
 
     case 169:			/* --end-of-line=TYPE           */
@@ -938,17 +928,11 @@ main (int argc, char *argv[])
 
   /* Name under which this program was called. */
   program_name = base_name (argv[0]);
-  program_invocation_name = xstrdup (program_name);
-  version_etc_copyright = N_("\
-Copyright (c) 1988-1993 Miguel Santana\n\
-Copyright (c) 1995-2000 Akim Demaille, Miguel Santana\n\
-Copyright (c) 2007- Akim Demaille, Miguel Santana and Masayuki Hatta");
+  /* program_invocation_name = xstrdup (program_name); */
 
   /* Set the NLS on */
   setlocale (LC_TIME, "");
-#ifdef HAVE_LC_MESSAGES
   setlocale (LC_MESSAGES, "");
-#endif
   setlocale (LC_CTYPE, "");
 
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -958,8 +942,7 @@ Copyright (c) 2007- Akim Demaille, Miguel Santana and Masayuki Hatta");
      or --version. */
   parse_long_options (argc, argv,
 		      NULL, GNU_PACKAGE, VERSION,
-		      "Akim Demaille, Miguel Santana",
-		      usage);
+		      usage, "Akim Demaille", "Miguel Santana", (char const *) NULL);
 
   /* Catch the exits and signals to cleanup the mess.
 
@@ -967,7 +950,6 @@ Copyright (c) 2007- Akim Demaille, Miguel Santana and Masayuki Hatta");
      find it beautiful to see `received signal blah blah' even if it
      is even before a2ps could make a move.  */
   atexit (exit_handler);
-  signame_init ();
 #define signal_set(Sig, Handler)		\
  do {						\
    if (signal (Sig, Handler) == SIG_IGN)	\
@@ -993,7 +975,7 @@ Copyright (c) 2007- Akim Demaille, Miguel Santana and Masayuki Hatta");
   /* Prepare to receive in the hash table of the delegations */
   delegation_table = delegation_table_new ();
 
-  /* Set the syntax that has be chosen for regexp */
+  /* Set the a2ps-specific regex syntax */
   re_set_syntax (RE_SYNTAX_A2PS);
 
   /* The spine of a2ps */
@@ -1039,10 +1021,10 @@ Copyright (c) 2007- Akim Demaille, Miguel Santana and Masayuki Hatta");
       /* Act like file(1) does: report guessed ssh key */
       if (argn < argc)
 	for (; argn < argc; argn++)
-	  guess ((uchar *) argv[argn]);
+	  guess (argv[argn]);
       else
 	/* A guess is asked upon stdin */
-	guess (UNULL);
+	guess (NULL);
       break;
 
       /* FIXME: for expand, which, and glob, should we give an error
@@ -1053,7 +1035,7 @@ Copyright (c) 2007- Akim Demaille, Miguel Santana and Masayuki Hatta");
       for (; argn < argc; argn++)
 	{
 	  fputs (expand_user_string (job, FIRST_FILE (job),
-				     "--list=expand", (uchar *) argv[argn]),
+				     "--list=expand", argv[argn]),
 		 stdout);
 	  putc ('\n', stdout);
 	}
@@ -1065,7 +1047,7 @@ Copyright (c) 2007- Akim Demaille, Miguel Santana and Masayuki Hatta");
       for (; argn < argc; argn++)
 	{
 	  char *cp;
-	  cp = pw_find_file (job->common.path, (uchar *) argv[argn], NULL);
+	  cp = pw_find_file (job->common.path, argv[argn], NULL);
 	  if (cp)
 	    {
 	      fputs (cp, stdout);
@@ -1078,16 +1060,16 @@ Copyright (c) 2007- Akim Demaille, Miguel Santana and Masayuki Hatta");
       /* Glob the arguments in the library, and report the full
          paths. */
       for (; argn < argc; argn++)
-	pw_glob_print (job->common.path, (uchar *) argv[argn], stdout);
+	pw_glob_print (job->common.path, argv[argn], stdout);
       break;
 
     case b_version:
       version_etc (stdout, NULL, GNU_PACKAGE, VERSION,
-		   "Akim Demaille, Miguel Santana");
+		   "Akim Demaille", "Miguel Santana", NULL);
       break;
 
     case b_help:
-      usage (0);
+      usage (EXIT_SUCCESS);
       break;
 
     case b_list_options:
@@ -1164,13 +1146,13 @@ Copyright (c) 2007- Akim Demaille, Miguel Santana and Masayuki Hatta");
 	a2ps_open_output_session (job);
 
 	if (argn == argc)	/* Print stdin */
-	  print (UNULL, &native_jobs, &delegated_jobs);
+	  print (NULL, &native_jobs, &delegated_jobs);
 	else			/* Print following files */
 	  for (; argn < argc; argn++)
-	    print ((uchar *) argv[argn], &native_jobs, &delegated_jobs);
+	    print (argv[argn], &native_jobs, &delegated_jobs);
 
 	if (!IS_EMPTY (toc))
-	  print_toc ((uchar *) _("Table of Content"), toc, &native_jobs);
+	  print_toc (_("Table of Content"), toc, &native_jobs);
 
 	if ((native_jobs == 0) && (delegated_jobs == 1))
 	  {
@@ -1222,7 +1204,6 @@ Copyright (c) 2007- Akim Demaille, Miguel Santana and Masayuki Hatta");
       abort ();
     }
 
-  a2ps_job_free (job);
   job = NULL;
 
   return (EXIT_SUCCESS);
