@@ -1,38 +1,35 @@
-/*
- * select.c -- Selection of a style sheet
- * Copyright (c) 1995-99 Akim Demaille, Miguel Santana
- *
- */
+/* select.c - selection of a style sheet
+   Copyright 1995-2017 Free Software Foundation, Inc.
 
-/*
- * This file is part of a2ps.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; see the file COPYING.  If not, write to
- * the Free Software Foundation, 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- */
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3, or (at your option)
+   any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
+   02110-1301, USA.  */
+
+#include <config.h>
+
+#include <fnmatch.h>
 
 #include "a2ps.h"
 #include "select.h"
 #include "routines.h"
-#include "xfnmatch.h"
 #include "getshline.h"
-#include "path-concat.h"
+#include "filenamecat.h"
 #include "pathwalk.h"
 #include "filtdir.h"
 #include "message.h"
 #include "quotearg.h"
+#include "xstrndup.h"
 
 extern char * style_request;
 
@@ -41,7 +38,7 @@ extern char * style_request;
  */
 extern a2ps_job * job;
 extern struct darray * sheets_map;
-
+
 
 /*
  * The user has requested the style sheet him self.
@@ -70,7 +67,7 @@ struct pattern_rule *
 pattern_rule_new (const char *pattern, int on_file_verdict, bool insensitive_p,
 		  const char *command)
 {
-  NEW (struct pattern_rule, res);
+  struct pattern_rule * res = XMALLOC (struct pattern_rule);
 
   res->pattern = pattern;
   res->on_file_verdict = on_file_verdict;
@@ -120,21 +117,19 @@ sheets_map_load_main (void)
   if (!file)
     {
       error (0, errno, _("cannot find file `%s'"), "sheets.map");
-      /* sheets.map can not be found: there is no automatic prettyprinting */
+      /* sheets.map cannot be found: there is no automatic prettyprinting */
       error (0, 0, _("automatic style selection cancelled"));
       return 0;
     }
 
   sheets_map_load (file);
 
-  free (file);
   return 1;
 }
-
-/* escapes the name of a file so that the shell groks it in 'single' q.marks.
-   The resulting pointer has to be free()ed when not longer used. */
-char *
-shell_escape(const char *fn)
+
+/* escapes the name of a file so that the shell groks it in 'single' q.marks. */
+static char *
+shell_escape (const char *fn)
 {
   size_t len = 0;
   const char *inp;
@@ -160,12 +155,12 @@ shell_escape(const char *fn)
 
   return retval;
 }
-
+
 /* What says file about the type of a file (result is malloc'd).  NULL
   if could not be run.  */
 
 static char *
-file_verdict_on (const uchar *filename)
+file_verdict_on (const char *filename_)
 {
   char *cp = NULL, * command;
   char buf [1024];
@@ -174,13 +169,13 @@ file_verdict_on (const uchar *filename)
   if (IS_EMPTY (job->file_command))
     return NULL;
 
-  filename = shell_escape(filename);
-  if(filename == NULL)
+  char *filename = shell_escape (filename_);
+  if (filename == NULL)
     return NULL;
   /* Call file(1) with the correct option */
   command = ALLOCA (char, (4
 			   + strlen (job->file_command)
-			   + ustrlen (filename)));
+			   + strlen (filename)));
   sprintf (command, "%s '%s'", job->file_command, (const char *) filename);
   free(filename);
   message (msg_tool, (stderr, "Reading pipe: `%s'\n", command));
@@ -196,8 +191,13 @@ file_verdict_on (const uchar *filename)
     }
 
   /* Get the answer */
-  fgets (buf, sizeof (buf), file_out);
+  char * res = fgets (buf, sizeof (buf), file_out);
   pclose (file_out);
+  if (res == NULL)
+    {
+      error (0, errno, _("error running file(!)"));
+      return NULL;
+    }
   message (msg_tool, (stderr, "file(1): %s", buf));
 
   /* File is expected to answer:
@@ -225,11 +225,10 @@ file_verdict_on (const uchar *filename)
  */
 #define rule(_i_) ((struct pattern_rule *)sheets_map->content[_i_])
 const char *
-get_command (const uchar *name_to_match, const uchar *name_to_file)
+get_command (const char *name_to_match, const char *name_to_file)
 {
-  int i;
   char *file_verdict;
-  uchar *name_to_match_lc;
+  char *name_to_match_lc;
 
   /* We only want to read the sheets map if needed, hence,
    * from here (not needed if the sheet name is given by the
@@ -253,31 +252,26 @@ get_command (const uchar *name_to_match, const uchar *name_to_file)
    * Christian Mondrup <scancm@biobase.dk> and Jens Henrik Leonhard
    * Jensen recjhl@mediator.uni-c.dk) */
   /* The loop is split to speed up */
-  for (i = sheets_map->len - 1 ; i >= 0 ; i--)
-    if (rule(i)->on_file_verdict)
-      {
-	/* Testing upon file's result */
-	if (file_verdict
-	    && !fnmatch (rule(i)->pattern, file_verdict, 0))
-	  {
-	    free (file_verdict);
-	    return rule(i)->command;
-	  }
-      }
-    else
-      {
-	/* Upon file name */
-	if (name_to_match
-	    && !fnmatch (rule(i)->pattern,
-			 (char *) (rule(i)->insensitive_p
-				   ? name_to_match_lc : name_to_match),
-			 0))
-	  {
-	    XFREE (file_verdict);
-	    return rule(i)->command;
-	  }
-      }
-  XFREE (file_verdict);
+  for (size_t i = sheets_map->len ; i-- > 0;)
+    {
+      if (rule(i)->on_file_verdict)
+        {
+          /* Testing upon file's result */
+          if (file_verdict
+              && !fnmatch (rule(i)->pattern, file_verdict, 0))
+            return rule(i)->command;
+        }
+      else
+        {
+          /* Upon file name */
+          if (name_to_match
+              && !fnmatch (rule(i)->pattern,
+                           (char *) (rule(i)->insensitive_p
+                                     ? name_to_match_lc : name_to_match),
+                           0))
+            return rule(i)->command;
+        }
+    }
 
   return "plain";
 }
